
#include "convert.h"

void debug_flush(void) {
  sysassert(!ferror(debug));
  sysassert(!fflush(debug));
}

const char *get_vardir(void) { return "."; }

static enum {
  mf_findwindow=      0001,
  mf_screenshot=      0010,
  mf_readscreenshot=  0020,
  mf_analyse=         0100,
  
  mode_findwindow=    0001,
  mode_screenshot=    0011,
  mode_analyse=       0120,

  mode_all=           0111,
} o_mode= mode_all;

static char *o_screenshots_fn;
static int o_single_page, o_quiet;

FILE *screenshots_file;


static void badusage(const char *fmt, ...)
     __attribute__((format(printf,1,2),noreturn));
static void badusage(const char *fmt, ...) {
  va_list al;
  va_start(al,fmt);
  exit(12);
}

static void open_screenshots_file(const char *mode) {
  screenshots_file= fopen(o_screenshots_fn, mode);
  if (!screenshots_file)
    fatal("could not open screenshots file `%s': %s",
	  o_screenshots_fn, strerror(errno));
}

int main(int argc, char **argv) {
  const char *arg;
  int r;

#define ARGVAL  ((*++argv) ? *argv : \
		 badusage("missing value for option %s",arg),(char*)0)

  while ((arg=*++argv)) {
    if (!strcmp(arg,"--find-window-only"))
      o_mode= mode_findwindow;
    else if (!strcmp(arg,"--screenshot-only"))
      o_mode= mode_screenshot;
    else if (!strcmp(arg,"--analyse-only"))
      o_mode= mode_analyse;
    else if (!strcmp(arg,"--single-page"))
      o_single_page= 1;
    else if (!strcmp(arg,"--quiet"))
      o_quiet= 1;
    else if (!strcmp(arg,"--screenshots-file"))
      o_screenshots_fn= ARGVAL;
#define DF(f)					\
    else if (!strcmp(arg,"-D" #f))		\
      debug_flags |= dbg_##f;
    DEBUG_FLAG_LIST
#undef DF
    else if (!strcmp(arg,"--window-id")) {
      char *ep;
      unsigned long windowid= strtoul(ARGVAL,&ep,0);
      if (*ep) badusage("invalid window id");
      set_yppclient_window(windowid);
    } else
      badusage("unknown option `%s'",arg);
  }
  
  if (!o_screenshots_fn) {
    r= asprintf(&o_screenshots_fn,"%s/#pages#.ppm",get_vardir());
    sysassert(r>=0);
  }

  if (o_mode & mf_findwindow) {
    screenshot_startup();
    find_yppclient_window();
  }
  if (o_mode & mf_screenshot) {
    open_screenshots_file("w");
    if (o_single_page) take_one_screenshot();
    else take_screenshots();
  }
  if (o_mode & mf_readscreenshot) {
    open_screenshots_file("r");
    if (o_single_page) read_one_screenshot();
    else read_screenshots();
  }
  if (o_mode & mf_analyse) {
    analyse();
    //output_tsv();
  }
  return 0;
}




DEFINE_VWRAPPERF(, progress, )
DEFINE_VWRAPPERF(, progress_log, )
DEFINE_VWRAPPERF(, progress_spinner, )
DEFINE_VWRAPPERF(, warning, )
DEFINE_VWRAPPERF(, fatal, NORET)

static int last_progress_len;
     
static void vprogress_core(int spinner, const char *fmt, va_list al) {
  int r;
  
  if (o_quiet) return;
  if (!isatty(2)) return;
  
  if (last_progress_len)
    putc('\r',stderr);

  r= vfprintf(stderr,fmt,al);

  if (spinner) {
    putc(spinner,stderr);
    r++;
  }

  if (r < last_progress_len) {
    fprintf(stderr,"%*s", last_progress_len - r, "");
    if (!r) putc('\r', stderr);
    else while (last_progress_len-- > r) putc('\b',stderr);
  }
  last_progress_len= r;

  if (ferror(stderr) || fflush(stderr)) _exit(16);
}
   
void vprogress(const char *fmt, va_list al) { vprogress_core(0,fmt,al); }
void vprogress_spinner(const char *fmt, va_list al) {
  static const char spinchars[]="/-\\";
  static int spinner;

  vprogress_core(spinchars[spinner],fmt,al);
  spinner++;
  spinner %= (sizeof(spinchars)-1);
}

void vprogress_log(const char *fmt, va_list al) {
  if (o_quiet) return;
  
  progress("");
  vfprintf(stderr,fmt,al);
  putc('\n',stderr);
  fflush(stderr);
}

void vwarning(const char *fmt, va_list al) {
  progress("");
  fputs("Warning: ",stderr);
  vfprintf(stderr,fmt,al);
  fputs("\n",stderr);
  fflush(stderr);
}

void vfatal(const char *fmt, va_list al) {
  progress("");
  fputs("\n\nFatal error: ",stderr);
  vfprintf(stderr,fmt,al);
  fflush(stderr);
  fputs("\n\n",stderr);
  _exit(4);
}

void sysassert_fail(const char *file, int line, const char *what) {
  int e= errno;
  progress("");
  fprintf(stderr,
	  "\nfatal operational error:\n"
	  " unsuccessful execution of: %s\n"
	  " %s:%d: %s\n\n",
	  what, file,line, strerror(e));
  _exit(16);
}

void *mmalloc(size_t sz) {
  void *r;
  if (!sz) return 0;
  sysassert( r= malloc(sz) );
  return r;
}
void *mrealloc(void *p, size_t sz) {
  assert(sz);
  void *r;
  sysassert( r= realloc(p,sz) );
  return r;
}
